@@ -1,17 +1,16 @@
 """Convert specified property values into computed values."""
 
 from math import pi
-from urllib.parse import unquote
 
-from tinycss2.color3 import parse_color
+from tinycss2.color5 import parse_color
 
 from ..logger import LOGGER
-from ..text.ffi import ffi, pango, units_to_double
-from ..text.line_break import Layout, first_line_metrics
-from ..urls import get_link_attribute
+from ..text.line_break import strut
+from ..urls import get_link_attribute, get_url_tuple
+from .functions import check_math
 from .properties import INITIAL_VALUES, ZERO_PIXELS, Dimension
-from .utils import (
-    ANGLE_TO_RADIANS, LENGTH_UNITS, LENGTHS_TO_PIXELS, safe_urljoin)
+from .units import ANGLE_TO_RADIANS, LENGTH_UNITS, to_pixels
+from .validation import validate_non_shorthand
 
 # Value in pixels of font-size for <absolute-size> keywords: 12pt (16px) for
 # medium, and scaling factors given in CSS3 for others:
@@ -123,35 +122,13 @@ assert all(width.value < height.value for width, height in PAGE_SIZES.values())
 
 INITIAL_PAGE_SIZE = PAGE_SIZES['a4']
 INITIAL_VALUES['size'] = tuple(
-    size.value * LENGTHS_TO_PIXELS[size.unit] for size in INITIAL_PAGE_SIZE)
+    to_pixels(size, None, 'size') for size in INITIAL_PAGE_SIZE)
 
 
 # Maps property names to functions returning the computed values
 COMPUTER_FUNCTIONS = {}
 
 
-def _font_style_cache_key(style, include_size=False):
-    key = str((
-        style['font_family'],
-        style['font_style'],
-        style['font_stretch'],
-        style['font_weight'],
-        style['font_variant_ligatures'],
-        style['font_variant_position'],
-        style['font_variant_caps'],
-        style['font_variant_numeric'],
-        style['font_variant_alternates'],
-        style['font_variant_east_asian'],
-        style['font_feature_settings'],
-        style['font_variation_settings'],
-        style['font_language_override'],
-        style['lang'],
-    ))
-    if include_size:
-        key += str(style['font_size']) + str(style['line_height'])
-    return key
-
-
 def register_computer(name):
     """Decorator registering a property ``name`` for a function."""
     name = name.replace('-', '_')
@@ -164,7 +141,8 @@ def register_computer(name):
 
 
 def compute_attr(style, values):
-    # TODO: use real token parsing instead of casting with Python types
+    # TODO: use real token parsing instead of casting with Python types, and follow new
+    # syntax. See https://drafts.csswg.org/css-values-5/#attr-notation.
     func_name, value = values
     assert func_name == 'attr()'
     attr_name, type_or_unit, fallback = value
@@ -173,13 +151,9 @@ def compute_attr(style, values):
         if type_or_unit == 'string':
             pass  # Keep the string
         elif type_or_unit == 'url':
-            if attr_value.startswith('#'):
-                attr_value = ('internal', unquote(attr_value[1:]))
-            else:
-                attr_value = (
-                    'external', safe_urljoin(style.base_url, attr_value))
+            attr_value = get_url_tuple(attr_value, style.base_url)
         elif type_or_unit == 'color':
-            attr_value = parse_color(attr_value.strip())
+            attr_value = parse_color(attr_value.strip(), style['color_scheme'])
         elif type_or_unit == 'integer':
             attr_value = int(attr_value.strip())
         elif type_or_unit == 'number':
@@ -193,6 +167,8 @@ def compute_attr(style, values):
         elif type_or_unit in ANGLE_TO_RADIANS:
             attr_value = Dimension(float(attr_value.strip()), type_or_unit)
             type_or_unit = 'angle'
+        else:
+            return
     except Exception:
         return
     return (type_or_unit, attr_value)
@@ -206,15 +182,30 @@ def background_image(style, name, values):
             value.stop_positions = tuple(
                 length(style, name, pos) if pos is not None else None
                 for pos in value.stop_positions)
+            value.color_hints = tuple(
+                length(style, name, hint) if hint is not None else None
+                for hint in value.color_hints)
         if type_ == 'radial-gradient':
             value.center, = compute_position(
                 style, name, (value.center,))
             if value.size_type == 'explicit':
-                value.size = length_or_percentage_tuple(
-                    style, name, value.size)
+                value.size = length_or_percentage_tuple(style, name, value.size)
     return values
 
 
+@register_computer('color')
+@register_computer('background-color')
+@register_computer('border-top-color')
+@register_computer('border-right-color')
+@register_computer('border-bottom-color')
+@register_computer('border-left-color')
+@register_computer('column-rule-color')
+@register_computer('outline-color')
+@register_computer('text-decoration-color')
+def color(style, name, values):
+    return parse_color(values, style['color_scheme'])
+
+
 @register_computer('background-position')
 @register_computer('object-position')
 def compute_position(style, name, values):
@@ -236,8 +227,7 @@ def length_or_percentage_tuple(style, name, values):
 @register_computer('clip')
 def length_tuple(style, name, values):
     """Compute the properties with a list of lengths."""
-    return tuple(
-        length(style, name, value, pixels_only=True) for value in values)
+    return tuple(length(style, name, value, pixels_only=True) for value in values)
 
 
 @register_computer('break-after')
@@ -268,38 +258,20 @@ def break_before_after(style, name, value):
 @register_computer('text-indent')
 @register_computer('hyphenate-limit-zone')
 @register_computer('flex-basis')
+@register_computer('text-underline-offset')
+@register_computer('text-decoration-thickness')
 def length(style, name, value, font_size=None, pixels_only=False):
     """Compute a length ``value``."""
-    if value in ('auto', 'content'):
+    if value in ('auto', 'content', 'from-font') or check_math(value):
         return value
-    if value.value == 0:
+    elif value.value == 0:
         return 0 if pixels_only else ZERO_PIXELS
-
-    unit = value.unit
-    if unit == 'px':
-        return value.value if pixels_only else value
-    elif unit in LENGTHS_TO_PIXELS:
-        # Convert absolute lengths to pixels
-        result = value.value * LENGTHS_TO_PIXELS[unit]
-    elif unit in ('em', 'ex', 'ch', 'rem'):
-        if font_size is None:
-            font_size = style['font_size']
-        if unit == 'ex':
-            # TODO: use context to use @font-face fonts
-            ratio = character_ratio(style, 'x')
-            result = value.value * font_size * ratio
-        elif unit == 'ch':
-            ratio = character_ratio(style, '0')
-            result = value.value * font_size * ratio
-        elif unit == 'em':
-            result = value.value * font_size
-        elif unit == 'rem':
-            result = value.value * style.root_style['font_size']
-    else:
+    elif value.unit not in LENGTH_UNITS:
         # A percentage or 'auto': no conversion needed.
         return value
 
-    return result if pixels_only else Dimension(result, 'px')
+    pixels = to_pixels(value, style, name, font_size)
+    return pixels if pixels_only else Dimension(pixels, 'px')
 
 
 @register_computer('bleed-left')
@@ -309,16 +281,14 @@ def length(style, name, value, font_size=None, pixels_only=False):
 def bleed(style, name, value):
     if value == 'auto':
         return Dimension(8 if 'crop' in style['marks'] else 0, 'px')
-    else:
-        return length(style, name, value)
+    return length(style, name, value)
 
 
 @register_computer('letter-spacing')
 def pixel_length(style, name, value):
     if value == 'normal':
         return value
-    else:
-        return length(style, name, value, pixels_only=True)
+    return length(style, name, value, pixels_only=True)
 
 
 @register_computer('background-size')
@@ -336,7 +306,7 @@ def image_orientation(style, name, values):
     if values in ('none', 'from-image'):
         return values
     angle, flip = values
-    return (int(round(angle / pi * 2)) % 4 * 90, flip)
+    return (round(angle / pi * 2) % 4 * 90, flip)
 
 
 @register_computer('border-top-width')
@@ -362,9 +332,77 @@ def border_width(style, name, value):
     return length(style, name, value, pixels_only=True)
 
 
+@register_computer('border-image-slice')
+@register_computer('mask-border-slice')
+def border_image_slice(style, name, values):
+    """Compute the ``border-image-slice`` property."""
+    computed_values = []
+    fill = None
+    for value in values:
+        if value == 'fill':
+            fill = value
+        else:
+            number, unit = value
+            if unit is None:
+                computed_values.append(number)
+            else:
+                computed_values.append(Dimension(number, '%'))
+    if len(computed_values) == 1:
+        computed_values *= 4
+    elif len(computed_values) == 2:
+        computed_values *= 2
+    elif len(computed_values) == 3:
+        computed_values.append(computed_values[1])
+    return (*computed_values, fill)
+
+
+@register_computer('border-image-width')
+@register_computer('mask-border-width')
+def border_image_width(style, name, values):
+    """Compute the ``border-image-width`` property."""
+    computed_values = []
+    for value in values:
+        if value == 'auto':
+            computed_values.append(value)
+        else:
+            number, unit = value
+            computed_values.append(number if unit is None else value)
+    if len(computed_values) == 1:
+        computed_values *= 4
+    elif len(computed_values) == 2:
+        computed_values *= 2
+    elif len(computed_values) == 3:
+        computed_values.append(computed_values[1])
+    return tuple(computed_values)
+
+
+@register_computer('border-image-outset')
+@register_computer('mask-border-outset')
+def border_image_outset(style, name, values):
+    """Compute the ``border-image-outset`` property."""
+    computed_values = [
+        value if isinstance(value, (int, float)) else length(style, name, value)
+        for value in values]
+    if len(computed_values) == 1:
+        computed_values *= 4
+    elif len(computed_values) == 2:
+        computed_values *= 2
+    elif len(computed_values) == 3:
+        computed_values.append(computed_values[1])
+    return tuple(computed_values)
+
+
+@register_computer('border-image-repeat')
+@register_computer('mask-border-repeat')
+def border_image_repeat(style, name, values):
+    """Compute the ``border-image-repeat`` property."""
+    return (values * 2) if len(values) == 1 else values
+
+
 @register_computer('column-width')
-def column_width(style, name, value):
-    """Compute the ``column-width`` property."""
+@register_computer('outline-offset')
+def length_pixels_only(style, name, value):
+    """Compute a pixel length property."""
     return length(style, name, value, pixels_only=True)
 
 
@@ -378,11 +416,10 @@ def border_radius(style, name, values):
 
 
 @register_computer('column-gap')
-def column_gap(style, name, value):
-    """Compute the ``column-gap`` property."""
-    if value == 'normal':
-        value = Dimension(1, 'em')
-    return length(style, name, value, pixels_only=True)
+@register_computer('row-gap')
+def gap(style, name, value):
+    """Compute the ``*-gap`` properties."""
+    return value if value == 'normal' else length(style, name, value)
 
 
 def _content_list(style, values):
@@ -408,7 +445,7 @@ def _content_list(style, values):
                 if attr is None:
                     computed_value = None
                 else:
-                    computed_value = (value[0], ((attr,) + value[1][1:]))
+                    computed_value = (value[0], (attr, *value[1][1:]))
             else:
                 computed_value = value
         if computed_value is None:
@@ -453,8 +490,7 @@ def display(style, name, value):
     # See https://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo.
     float_ = style.specified['float']
     position = style.specified['position']
-    if position in ('absolute', 'fixed') or float_ != 'none' or (
-            style.is_root_element):
+    if position in ('absolute', 'fixed') or float_ != 'none' or style.is_root_element:
         if value == ('inline-table',):
             return ('block', 'table')
         elif len(value) == 1 and value[0].startswith('table-'):
@@ -502,7 +538,7 @@ def font_size(style, name, value):
                 return keyword_values[-i - 1]
         else:
             return parent_font_size * 0.8
-    elif value.unit == '%':
+    elif isinstance(value, Dimension) and value.unit == '%':
         return value.value * parent_font_size / 100
     else:
         return length(
@@ -527,6 +563,72 @@ def font_weight(style, name, value):
         return value
 
 
+def _compute_track_breadth(style, name, value):
+    """Compute track breadth."""
+    if value in ('auto', 'min-content', 'max-content'):
+        return value
+    elif isinstance(value, Dimension):
+        if value.unit and value.unit.lower() == 'fr':
+            return value
+        else:
+            return length(style, name, value)
+
+
+def _track_size(style, name, values):
+    """Compute track size."""
+    return_values = []
+    for i, value in enumerate(values):
+        if i % 2 == 0:
+            # line name
+            return_values.append(value)
+        else:
+            # track section
+            track_breadth = _compute_track_breadth(style, name, value)
+            if track_breadth:
+                return_values.append(track_breadth)
+            elif value[0] == 'minmax()':
+                return_values.append((
+                    'minmax()',
+                    _compute_track_breadth(style, name, value[1]),
+                    _compute_track_breadth(style, name, value[2])))
+            elif value[0] == 'fit-content()':
+                return_values.append((
+                    'fit-content()', length(style, name, value[1])))
+            elif value[0] == 'repeat()':
+                return_values.append((
+                    'repeat()', value[1], _track_size(style, name, value[2])))
+    return tuple(return_values)
+
+
+@register_computer('grid-template-columns')
+@register_computer('grid-template-rows')
+def grid_template(style, name, values):
+    """Compute the ``grid-template-*`` properties."""
+    if values == 'none' or values[0] == 'subgrid':
+        return values
+    else:
+        return _track_size(style, name, values)
+
+
+@register_computer('grid-auto-columns')
+@register_computer('grid-auto-rows')
+def grid_auto(style, name, values):
+    """Compute the ``grid-auto-*`` properties."""
+    return_values = []
+    for value in values:
+        track_breadth = _compute_track_breadth(style, name, value)
+        if track_breadth:
+            return_values.append(track_breadth)
+        elif value[0] == 'minmax()':
+            return_values.append((
+                'minmax()', grid_auto(style, name, [value[1]])[0],
+                grid_auto(style, name, [value[2]])[0]))
+        elif value[0] == 'fit-content()':
+            return_values.append((
+                'fit-content()', grid_auto(style, name, [value[1]])[0]))
+    return tuple(return_values)
+
+
 @register_computer('line-height')
 def line_height(style, name, value):
     """Compute the ``line-height`` property."""
@@ -556,26 +658,23 @@ def anchor(style, name, values):
 def link(style, name, values):
     """Compute the ``link`` property."""
     if values == 'none':
-        return None
-    else:
-        type_, value = values
-        if type_ == 'attr()':
-            return get_link_attribute(style.element, value, style.base_url)
-        else:
-            return values
+        return
+    type_, value = values
+    if type_ == 'attr()':
+        return get_link_attribute(style.element, value, style.base_url)
+    return values
 
 
 @register_computer('lang')
 def lang(style, name, values):
     """Compute the ``lang`` property."""
     if values == 'none':
-        return None
-    else:
-        name, key = values
-        if name == 'attr()':
-            return style.element.get(key) or None
-        elif name == 'string':
-            return key
+        return
+    name, key = values
+    if name == 'attr()':
+        return style.element.get(key) or None
+    elif name == 'string':
+        return key
 
 
 @register_computer('tab-size')
@@ -598,17 +697,24 @@ def transform(style, name, value):
 @register_computer('vertical-align')
 def vertical_align(style, name, value):
     """Compute the ``vertical-align`` property."""
+    from ..css import resolve_math
+
     # Use +/- half an em for super and sub, same as Pango.
     # (See the SUPERSUB_RISE constant in pango-markup.c)
-    if value in (
-            'baseline', 'middle', 'text-top', 'text-bottom', 'top', 'bottom'):
+    if check_math(value):
+        height, _ = strut(style)
+        result = resolve_math(value, style, 'vertical_align', height)
+        value = validate_non_shorthand((result,), 'vertical-align')[0][1]
+        if value is None:
+            value = 'baseline'
+    if value in ('baseline', 'middle', 'text-top', 'text-bottom', 'top', 'bottom'):
         return value
     elif value == 'super':
         return style['font_size'] * 0.5
     elif value == 'sub':
         return style['font_size'] * -0.5
     elif value.unit == '%':
-        height, _ = strut_layout(style)
+        height, _ = strut(style)
         return height * value.value / 100
     else:
         return length(style, name, value, pixels_only=True)
@@ -617,79 +723,4 @@ def vertical_align(style, name, value):
 @register_computer('word-spacing')
 def word_spacing(style, name, value):
     """Compute the ``word-spacing`` property."""
-    if value == 'normal':
-        return 0
-    else:
-        return length(style, name, value, pixels_only=True)
-
-
-def strut_layout(style, context=None):
-    """Return a tuple of the used value of ``line-height`` and the baseline.
-
-    The baseline is given from the top edge of line height.
-
-    """
-    if style['font_size'] == 0:
-        return 0, 0
-
-    if context:
-        key = _font_style_cache_key(style, include_size=True)
-        if key in context.strut_layouts:
-            return context.strut_layouts[key]
-
-    layout = Layout(context, style)
-    layout.set_text(' ')
-    line, _ = layout.get_first_line()
-    _, _, _, _, text_height, baseline = first_line_metrics(
-        line, '', layout, resume_at=None, space_collapse=False, style=style)
-    if style['line_height'] == 'normal':
-        result = text_height, baseline
-        if context:
-            context.strut_layouts[key] = result
-        return result
-    type_, line_height = style['line_height']
-    if type_ == 'NUMBER':
-        line_height *= style['font_size']
-    result = line_height, baseline + (line_height - text_height) / 2
-    if context:
-        context.strut_layouts[key] = result
-    return result
-
-
-def character_ratio(style, character):
-    """Return the ratio of 1ex/font_size or 1ch/font_size."""
-    # TODO: use context to use @font-face fonts
-
-    assert character in ('x', '0')
-
-    cache = style.cache[f'ratio_{"ex" if character == "x" else "ch"}']
-    cache_key = _font_style_cache_key(style)
-    if cache_key in cache:
-        return cache[cache_key]
-
-    # Avoid recursion for letter-spacing and word-spacing properties
-    style = style.copy()
-    style['letter_spacing'] = 'normal'
-    style['word_spacing'] = 0
-    # Random big value
-    style['font_size'] = 1000
-
-    layout = Layout(context=None, style=style)
-    layout.set_text(character)
-    line, _ = layout.get_first_line()
-
-    ink_extents = ffi.new('PangoRectangle *')
-    logical_extents = ffi.new('PangoRectangle *')
-    pango.pango_layout_line_get_extents(line, ink_extents, logical_extents)
-    if character == 'x':
-        measure = -units_to_double(ink_extents.y)
-    else:
-        measure = units_to_double(logical_extents.width)
-    ffi.release(ink_extents)
-    ffi.release(logical_extents)
-
-    # Zero means some kind of failure, fallback is 0.5.
-    # We round to try keeping exact values that were altered by Pango.
-    ratio = round(measure / style['font_size'], 5) or 0.5
-    cache[cache_key] = ratio
-    return ratio
+    return 0 if value == 'normal' else length(style, name, value, pixels_only=True)
